#!/bin/bash

###############################################################################
#                                                                             #
#                           /etc/acpi/action.sh                               #
#                               v2003.09.01                                   #
#                     written by Christopher Knoerle                          #
#                           mail@cknoerle.de                                  #
#                                                                             #
#                     called by acpid for all events                          #
#                                                                             #
###############################################################################

# The latest version is available from
# http://cknoerle.homelinux.org/notebook/

# This works with kernel 2.4.20-gentoo-r6 on my HP Compaq nx9000 Laptop
# For more information have a look at http://cknoerle.homelinux.org/nx9000

# This should be placed in /etc/acpi/action.sh and should be called by acpid.
# To make this happen you need the acpid installed an running.
# acpid can be obtained from http://acpid.sourceforge.net
# For this to work you need only one event handler in /etc/acpi/event
# It should contain something like the following:
#
# event=.*
# action=/etc/acpi/action.sh %e


# let's set some paths
LOGGER=/usr/bin/logger # logs to syslog - an echo logs to /var/log/acpid
XGAMMA=/usr/X11R6/bin/xgamma
XSET=/usr/X11R6/bin/xset
CPU_LIMIT=/proc/acpi/processor/CPU0/limit
BAT_STATE=/proc/acpi/battery/BAT1/state
AC_STATE=/proc/acpi/ac_adapter/ACAD/state


# if we didn't know about an event
# log to /var/log/acpid to do something like 'grep #### /var/log/acpid' :-)
no_action ()
{
   echo "#### NO ACTION FOR EVENT:" $*
   exit 1
}


# AC Adapter plugged in
# try to gain performance
acad_in ()
{
   $LOGGER "[acpid] AC Adapter plugged IN"

   if ps -A | grep -q X
   then
      # set gamma in X to 100%
      $XGAMMA -d 0:0 -gamma 1.000
      # shutdown display after 300s
      $XSET -d 0:0 dpms 0 0 300
   fi

   # set cpu performance state 0, throttling to 0
   echo -n 0:0 > $CPU_LIMIT
}


# AC Adapter plugged out
# try to save power
acad_out ()
{
   $LOGGER "[acpid] AC Adapter plugged OUT"

   if ps -A | grep -q X
   then
      # set gamma in X to 66.6%
      $XGAMMA -d 0:0 -gamma 0.666
      # shutdown display after 60s
      $XSET -d 0:0 dpms 0 0 60
   fi

   # set cpu performance state 1, throttling to 0
   echo -n 1:0 > $CPU_LIMIT
}


# state of battery changed
# shutdown if battery capacity is low
battery ()
{
   if grep discharging $BAT_STATE
   then
      if ((`awk '/remaining/ { print $3 }' $BAT_STATE` < 150))
      then
         $LOGGER "[acpid] battery capacity is low"
         btn_pwr
      fi
   fi
}


# powerbutton pressed
# if kde is running try to show dialog
# else shutdown immediately
btn_pwr ()
{
   $LOGGER "[acpid] Powerbutton pressed"

   if ps -Af | grep -q '[k]desktop' && test -f $KDEDIR/bin/dcop
   then
      $LOGGER "[acpid] KDE running: asking user what to do"
      dcop --all-sessions ksmserver ksmserver logout 1 2 0 && exit 0
   else
      $LOGGER "[acpid] shutdown initiated"
      /sbin/init 0
   fi
}

# lid closed
# The only thing useful about this event could be throttling the CPU down
# in case we are on battery - but how to get it to normal speed again since
# there is no event for open lid?
#
# OK, I've seen something like /proc/acpi/button/lid/LID/status
# which returns open or closed with Alan Cox's 2.4.22-rc2-r3
# that should be useful :-)
btn_lid ()
{
   $LOGGER "[acpid] Lid closed"

#   if grep -q on-line $AC_STATE
#   then
#      echo -n 0:0 > $CPU_LIMIT
#   else
#      echo -n 1:7 > $CPU_LIMIT
#   fi
}


# let's see which event occured and what action to take :-)
case "$*" in
   ac_adapter\ ACAD\ 00000080\ 00000000)
      acad_out
   ;;
   ac_adapter\ ACAD\ 00000080\ 00000001)
      acad_in
   ;;
   battery\ BAT1\ 00000080\ 00000001)
      battery
   ;;
   button?lid\ LID\ 00000080\ ????????) # the last eight digits count the times
      btn_lid                           # the lid was shut
   ;;
   button?power\ PWRF\ 00000080\ 00000001)
      btn_pwr
   ;;
   *)
      no_action
   ;;
esac

# EOF
